"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const where_builder_1 = require("./where.builder");
const query_builder_abstract_1 = require("./query-builder.abstract");
class FilterQueryBuilder extends query_builder_abstract_1.AbstractQueryBuilder {
    constructor(repo, whereBuilder = new where_builder_1.WhereBuilder(repo)) {
        super(repo);
        this.repo = repo;
        this.whereBuilder = whereBuilder;
    }
    select(query) {
        let qb = this.applyFilter(this.createQueryBuilder(), query.filter);
        qb = this.applySorting(qb, query.sorting);
        qb = this.applyPaging(qb, query.paging);
        return qb;
    }
    delete(query) {
        return this.applyFilter(this.repo.createQueryBuilder().delete(), query.filter);
    }
    update(query) {
        const qb = this.applyFilter(this.repo.createQueryBuilder().update(), query.filter);
        return this.applySorting(qb, query.sorting);
    }
    applyPaging(qb, paging) {
        if (!paging) {
            return qb;
        }
        return qb.limit(paging.limit).offset(paging.offset);
    }
    applyFilter(qb, filter) {
        if (!filter) {
            return qb;
        }
        return this.whereBuilder.build(qb, filter);
    }
    applySorting(qb, sorts) {
        if (!sorts) {
            return qb;
        }
        return sorts.reduce((prevQb, { field, direction, nulls }) => {
            return prevQb.addOrderBy(this.fieldToDbCol(field), direction, nulls);
        }, qb);
    }
    createQueryBuilder() {
        return this.repo.createQueryBuilder();
    }
}
exports.FilterQueryBuilder = FilterQueryBuilder;
//# sourceMappingURL=filter-query.builder.js.map