"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const sql_comparison_builder_1 = require("./sql-comparison.builder");
class WhereBuilder {
    constructor(repository, sqlComparisionBuilder = new sql_comparison_builder_1.SQLComparisionBuilder(repository)) {
        this.repository = repository;
        this.sqlComparisionBuilder = sqlComparisionBuilder;
    }
    build(where, filter) {
        const { and, or } = filter;
        if (and && and.length) {
            this.filterAnd(where, and);
        }
        if (or && or.length) {
            this.filterOr(where, or);
        }
        return this.filterFields(where, filter);
    }
    filterAnd(where, filter) {
        return filter.reduce((w, f) => w.andWhere(this.createBrackets(f)), where);
    }
    filterOr(where, filter) {
        return filter.reduce((w, f) => where.orWhere(this.createBrackets(f)), where);
    }
    createBrackets(filter) {
        return new typeorm_1.Brackets(qb => this.build(qb, filter));
    }
    filterFields(where, filter) {
        return Object.keys(filter).reduce((w, field) => {
            if (field !== 'and' && field !== 'or') {
                return this.withFilterComparison(where, field, this.getField(filter, field));
            }
            return w;
        }, where);
    }
    getField(obj, field) {
        return obj[field];
    }
    withFilterComparison(where, field, cmp) {
        return where.andWhere(new typeorm_1.Brackets(qb => {
            const opts = Object.keys(cmp);
            const sqlComparisons = opts.map(cmpType => this.sqlComparisionBuilder.build(field, cmpType, cmp[cmpType]));
            sqlComparisons.map(({ sql, params }) => qb.orWhere(sql, params));
        }));
    }
}
exports.WhereBuilder = WhereBuilder;
//# sourceMappingURL=where.builder.js.map