"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const class_transformer_1 = require("class-transformer");
const query_1 = require("../query");
class TypeormQueryService {
    constructor(repo, filterQueryBuilder = new query_1.FilterQueryBuilder(repo)) {
        this.repo = repo;
        this.filterQueryBuilder = filterQueryBuilder;
    }
    get entityType() {
        return this.repo.target;
    }
    get alias() {
        return this.repo.metadata.targetName;
    }
    async query(query) {
        return this.filterQueryBuilder.select(query).getMany();
    }
    async queryOne(query) {
        return this.filterQueryBuilder.select({ ...query, paging: { limit: 1, offset: 0 } }).getOne();
    }
    async getOne(query) {
        const entityOrUndefined = await this.queryOne(query);
        if (!entityOrUndefined) {
            throw new common_1.NotFoundException(`${this.alias} not found`);
        }
        return entityOrUndefined;
    }
    async findById(id) {
        return this.repo.findOne(id);
    }
    getById(id) {
        return this.repo.findOneOrFail(id);
    }
    createMany(create) {
        return this.repo.save(create.items.map(item => this.ensureIsEntity(item)));
    }
    createOne(create) {
        return this.repo.save(this.ensureIsEntity(create.item));
    }
    async deleteMany(deleteMany) {
        const deleteResult = await this.filterQueryBuilder.delete({ filter: deleteMany.filter }).execute();
        return { deletedCount: deleteResult.affected || 0 };
    }
    async deleteOne(deleteOne) {
        const entity = await this.getById(deleteOne.id);
        return this.repo.remove(entity);
    }
    async updateMany(update) {
        const updateResult = await this.filterQueryBuilder
            .update({ filter: update.filter })
            .set({ ...update.update })
            .execute();
        return { updatedCount: updateResult.affected || 0 };
    }
    async updateOne(update) {
        const entity = await this.getById(update.id);
        return this.repo.save(this.repo.merge(entity, update.update));
    }
    ensureIsEntity(obj) {
        if (obj instanceof this.entityType) {
            return obj;
        }
        return class_transformer_1.plainToClass(this.entityType, obj);
    }
}
exports.TypeormQueryService = TypeormQueryService;
//# sourceMappingURL=typeorm-query.service.js.map